
# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "authors": [
  "Padraic Shafer"
 ],
 "date": "2021-03-16T07:11:44-0700",
 "dirty": false,
 "error": null,
 "full-revisionid": "02f2f88927818f6a56f311b9d230be8938ae8f17",
 "version": "0.9.0"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
